import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:wowit/message.dart';
import 'package:wowit/methods.dart';
import 'package:progressive_image/progressive_image.dart';

class CommonWidget {
  const CommonWidget._();

  static Widget getWidgetImage(
    Map<String, Object> dataImage, {
    double? width,
    double? height,
    BoxFit fit = BoxFit.cover,
    double blur = 20,
    Duration fadeDuration = const Duration(milliseconds: 500),
    AlignmentGeometry alignment = Alignment.center,
    bool matchTextDirection = false,
    bool excludeFromSemantics = false,
    String? imageSemanticLabel,

    /// Khi fieldName != hinhAnh
    String? fieldName,
    Widget? errorBuilder,
  }) {
    String thumbnailUrl = Methods.getString(dataImage,
        fieldName != null ? fieldName + '_small_path' : 'hinhAnh_small_path');
    String imageUrl = Methods.getString(dataImage,
        fieldName != null ? fieldName + '_image_path' : 'hinhAnh_image_path');

    if (thumbnailUrl.isNotEmpty && imageUrl.isNotEmpty) {
      return ProgressiveImage.custom(
        //placeholder: const AssetImage('assets/images/default.jpg'),
        thumbnail: NetworkImage(Methods.getURL(thumbnailUrl)),
        image: NetworkImage(Methods.getURL(imageUrl)),
        height: height ?? double.infinity,
        width: width ?? double.infinity,
        fit: fit,
        blur: blur,
        fadeDuration: fadeDuration,
        alignment: alignment,
        matchTextDirection: matchTextDirection,
        excludeFromSemantics: excludeFromSemantics,
        imageSemanticLabel: imageSemanticLabel,
        placeholderBuilder: (_) =>
            errorBuilder ??
            Image.asset(
              'assets/images/default.jpg',
              width: width,
              height: height,
              fit: fit,
            ),
      );
    } else if (thumbnailUrl.isEmpty && imageUrl.isNotEmpty) {
      return Image.network(
        Methods.getURL(imageUrl),
        width: width,
        height: height,
        fit: fit,
        errorBuilder: (_, __, ___) =>
            errorBuilder ??
            Image.asset(
              'assets/images/default.jpg',
              width: width,
              height: height,
              fit: fit,
            ),
      );
    } else {
      return Image.asset(
        'assets/images/default.jpg',
        width: width,
        height: height,
        fit: fit,
      );
    }
  }

  static Widget getWidgetDienTich(
    double metter, {
    String mode = 'ha',
    TextStyle? textStyle,
  }) {
    double ha = Methods.round(metter / 10000.0, 2);
    String giaTri = (Methods.toInt(metter.toString())).toString();
    String donVi1 = Message.getMessage('M2');
    //String donVi2 = "2";
    if (ha > 0.1 && mode == 'ha') {
      giaTri = NumberFormat.decimalPattern().format(ha).toString();
      giaTri =
          giaTri.endsWith('.0') ? Methods.toInt(giaTri).toString() : giaTri;
      donVi1 = 'ha';
      //donVi2 = "";
    }

    var view = Row(
      children: [
        Text(
          giaTri,
          style: textStyle,
        ),
        Padding(
          padding: const EdgeInsets.only(bottom: 1),
          child: Text(
            donVi1,
            overflow: TextOverflow.ellipsis,
            textScaleFactor: 1.0,
            style: textStyle,
          ),
        ),
      ],
    );

    return view;
  }
}
